package org.icroco.tablemodel;

import javax.swing.JTable;

import org.icroco.tablemodel.blinking.IBlinkTableModel;
import org.icroco.tablemodel.impl.DefaultBeanTableModelFactory;
import org.icroco.tablemodel.impl.blinking.DefaultBeanBlinkTableModelFactory;

/**
 * {@link TableModelFactory} is an helper to build table model. In fact, it's
 * just a wrapper to another implementation because we have to use some package
 * protected method.
 * 
 * @author ouaibsky@gmail.com
 * 
 */
public final class TableModelFactory
{

	private TableModelFactory()
	{
		super();

	}

	/**
	 * This is the better way to get a {@link BeanTableModel}. We make some
	 * stuff like JTable optimization for flashing, install renderer wrapper.
	 * Warning set up your own renderer on your JTable before calling this
	 * method because we make some stuff concerning blinking or renderer
	 * annotation. ...
	 * 
	 * @param <B>
	 * @param aTable
	 * @param aClazz
	 * @return
	 * @throws BeanTableModelException
	 */
	public final static <B> IBlinkTableModel<B> createTableModel(final JTable aTable, final Class<B> aClazz) throws BeanTableModelException
	{
		return DefaultBeanBlinkTableModelFactory.createBlinkTableModel(aTable, aClazz);
	}

	public final static <B> IBeanTableModel<B> createTableModel(final Class<B> aClazz) throws BeanTableModelException
	{
		return DefaultBeanTableModelFactory.createTableModel(aClazz);
	}

}
