package org.icroco.tablemodel.impl;

import java.lang.reflect.Method;

import org.icroco.tablemodel.BeanTableModelException;



/**
 * An helper to get getter/setter compliant with JabaBean naing convention.
 * 
 * @version $Revision: 1.1 $
 * 
 */
public final class BeanHelper
{
    private BeanHelper()
    {
        super();
    }

    /**
     * 
     */
    private static final String START_STR  = "^";
    /**
     * 
     */
    private static final String EMPTY_STR  = "";
    /**
     * 
     */
    private static final String SET_PREFIX = "set"; //$NON-NLS-1$
    /**
     * 
     */
    private static final String GET_PREFIX = "get"; //$NON-NLS-1$
    /**
     * 
     */
    private static final String IS_PREFIX  = "is"; //$NON-NLS-1$

    public static boolean isGetter(final Method aMethod) throws BeanTableModelException
    {
        final String name = aMethod.getName();
        if (name.startsWith(GET_PREFIX) || name.startsWith(IS_PREFIX))
        {
            if (aMethod.isVarArgs() || aMethod.getParameterTypes().length > 0)
            {
                throw new BeanTableModelException("Getter based on bean naming convemtion must have 0 parameters, method: " + aMethod);
            }
            if (aMethod.getReturnType().isPrimitive())
            {
                throw new BeanTableModelException("Getter need to have non-primitive as return value" + aMethod);
            }
            return true;
        } else
        {
            throw new BeanTableModelException(
                    "You have to annotate getter based on bean naming convention: need to start by 'get' or 'is' prefix. method: "
                            + aMethod);
        }
    }

    public static final Method checkSetter(final Class<?> aClazz, final Method aGetter) throws BeanTableModelException
    {
        String baseName = aGetter.getName();
        if (baseName.startsWith(IS_PREFIX))
        {
            baseName = baseName.replaceFirst(IS_PREFIX, "");
        } else if (baseName.startsWith(GET_PREFIX))
        {
            baseName = baseName.replaceFirst(GET_PREFIX, "");
        }

        final Class<?> propType = aGetter.getReturnType();
        Method setter = null;
        try
        {
            setter = aClazz.getMethod(SET_PREFIX + baseName, propType);
        } catch (final SecurityException e)
        {
            throw new BeanTableModelException("Failed to find setter: set" + baseName + "(" + propType.getName() + ")", e);
        } catch (final NoSuchMethodException e)
        {
            throw new BeanTableModelException("Failed to find setter: set" + baseName + "(" + propType.getName()
                    + ") , check if parameter type is not a primitive type", e);
            // TODO replace by a generic setter wich dump method call.
        }

        return setter;
    }

    /**
     * @param aMethod
     * @return
     */
    public static boolean isSetter(final Method aMethod)
    {
        boolean lReturn = false;

        if (aMethod.getName().startsWith(SET_PREFIX))
            lReturn = true;

        return lReturn;

    }

    /**
     * This method
     * 
     * @param aMethod
     * @return
     */
    public static String getBeanName(final Method aMethod)
    {
        String lReturn = null;

        if (aMethod.getName().startsWith(IS_PREFIX))
            lReturn = aMethod.getName().replaceFirst(START_STR + IS_PREFIX, EMPTY_STR);
        else if (aMethod.getName().startsWith(GET_PREFIX))
            lReturn = aMethod.getName().replaceFirst(START_STR + GET_PREFIX, EMPTY_STR);
        else if (aMethod.getName().startsWith(SET_PREFIX))
            lReturn = aMethod.getName().replaceFirst(START_STR + SET_PREFIX, EMPTY_STR);

        return lReturn;
    }
}
