package org.icroco.tablemodel.impl;

import java.awt.EventQueue;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.concurrent.locks.ReadWriteLock;
import java.util.concurrent.locks.ReentrantReadWriteLock;

import javax.swing.table.AbstractTableModel;

import org.icroco.tablemodel.ABeanTableModel;
import org.icroco.tablemodel.ABeanTableModelProperty;
import org.icroco.tablemodel.BeanTableModelException;
import org.icroco.tablemodel.IBeanTableModel;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * This class wrap a benaobject into a JTable. Do not use directly this class,
 * in place use the helper {@link BeanTableModelHelper}
 * 
 * @see BeanTableModelHelper
 * @version $Revision: 1.8 $
 * 
 * @param <M>
 */
public class BeanTableModel<M>
		extends AbstractTableModel
		implements IBeanTableModel<M>
{
	/**
     * 
     */
	private static final int BUF_SIZE = 100;

	/**
	 * Logger for this class
	 */
	private transient static Logger logger = LoggerFactory.getLogger(BeanTableModel.class);

	/**
     * 
     */
	private static final int DEFAULT_LIST_SIZE = 50;
	/**
     * 
     */
	private static final long serialVersionUID = 1L;
	final private ArrayList<M> array;
	final private HashMap<M, Integer> arrayMap;
	private transient ReadWriteLock lock;
	private final Class<M> clazz;
	protected TableModelProperty properties[];
	private static final transient MessageFormat ERROR_1 = new MessageFormat(Messages.getString("BeanTableModel.1")); //$NON-NLS-1$
	private static final transient MessageFormat ERROR_2 = new MessageFormat(Messages.getString("BeanTableModel.2")); //$NON-NLS-1$
	private static final transient MessageFormat ERROR_3 = new MessageFormat(Messages.getString("BeanTableModel.3")); //$NON-NLS-1$
	private static final transient MessageFormat ERROR_4 = new MessageFormat(Messages.getString("BeanTableModel.4")); //$NON-NLS-1$
	private static final transient MessageFormat ERROR_5 = new MessageFormat(Messages.getString("BeanTableModel.5")); //$NON-NLS-1$
	private static final transient MessageFormat ERROR_6 = new MessageFormat(Messages.getString("BeanTableModel.6")); //$NON-NLS-1$
	private static final transient MessageFormat ERROR_7 = new MessageFormat(Messages.getString("BeanTableModel.7")); //$NON-NLS-1$
	private static final transient MessageFormat ERROR_8 = new MessageFormat(Messages.getString("BeanTableModel.8")); //$NON-NLS-1$
	private static final transient MessageFormat ERROR_9 = new MessageFormat(Messages.getString("BeanTableModel.9")); //$NON-NLS-1$
	private static final transient MessageFormat ERROR_10 = new MessageFormat(Messages.getString("BeanTableModel.10")); //$NON-NLS-1$
	private static final transient MessageFormat ERROR_12 = new MessageFormat(Messages.getString("BeanTableModel.12")); //$NON-NLS-1$

	// // Flashing
	// private transient JTable table;
	// private final transient Rectangle cacheRectangle = new Rectangle();

	/**
	 * Constructor.
	 * 
	 * @param aClass
	 * @throws BeanTableModelException
	 */
	public BeanTableModel(final Class<M> aClass) throws BeanTableModelException
	{
		super();
		this.lock = new EmptyLock();
		try
		{
			if (aClass == null)
			{
				throw new BeanTableModelException(Messages.getString("BeanTableModel.0")); //$NON-NLS-1$
			}
			this.clazz = aClass;
			this.lock = new EmptyLock();
			this.array = new ArrayList<M>(DEFAULT_LIST_SIZE);
			this.arrayMap = new HashMap<M, Integer>();
		}
		catch (final BeanTableModelException exception)
		{
			dumpProperties(this, true);
			throw exception;
		}
	}

	public final void parseAnnotation() throws BeanTableModelException
	{
		readAnnotationClass(this.clazz);
		final List<TableModelProperty> lProperties = readAnnotationMethod(this.clazz);
		Collections.sort(lProperties);
		checkSetter(lProperties);
		this.properties = lProperties.toArray(new TableModelProperty[lProperties.size() > 0 ? lProperties.size() - 1 : 0]);

		// dump info at end for debugging
		if (logger.isDebugEnabled())
		{
			dumpProperties(this, false);
		}
	}

	protected void readAnnotationClass(final Class<M> clazz2) throws BeanTableModelException
	{
		final ABeanTableModel annotation = clazz2.getAnnotation(ABeanTableModel.class);

		if (annotation != null)
		{
			if (annotation.isTreadSafe())
				this.lock = new ReentrantReadWriteLock();
		}

	}

	protected TableModelProperty createNewProperty()
	{
		return new TableModelProperty();
	}

	protected List<TableModelProperty> readAnnotationMethod(final Class<M> aClazz) throws BeanTableModelException
	{
		final Method methods[] = aClazz.getMethods();
		final List<TableModelProperty> lProperties = new ArrayList<TableModelProperty>(methods.length);
		final TableModelProperty search = createNewProperty();
		ABeanTableModelProperty aProp;
		String found;
		int idx;

		try
		{
			for (final Method method : methods)
			{
				aProp = method.getAnnotation(ABeanTableModelProperty.class);
				if (aProp != null)
				{
					if (BeanHelper.isGetter(method))
					{
						found = aProp.name() == null ? BeanHelper.getBeanName(method) : aProp.name();
						search.name = found;
						idx = lProperties.indexOf(search);
						if (idx < 0) // not found
						{
							final TableModelProperty prop = createNewProperty();
							prop.name = found;
							prop.label = found; // TODO: read from resource file
							prop.isEditable = aProp.isEditable();
							prop.columnClass = method.getReturnType();
							// prop.blink = aProp.isBlinkOnUpdate();
							// prop.blinkBgColor =
							// Color.decode(aProp.blinkBgColor());
							prop.orderKey = aProp.orderKey();
							prop.getter = method;
							final Method setter = BeanHelper.checkSetter(aClazz, method);
							if (setter != null)
								prop.setter = setter;
							lProperties.add(prop);
						}
						else
							throw new BeanTableModelException(ERROR_1.format(new Object[] { found }));
					}
					else
						throw new BeanTableModelException(ERROR_12.format(new Object[] {}));
				}
			}
		}
		catch (final BeanTableModelException exception)
		{
			throw exception;
		}
		finally
		{
			Collections.sort(lProperties);
			this.properties = lProperties.toArray(new TableModelProperty[lProperties.size() > 0 ? lProperties.size() - 1 : 0]);
		}

		return lProperties;
	}

	/**
	 * @param aClazz
	 * @param aProperties
	 * @throws BeanTableModelException
	 */
	private void checkSetter(final List<TableModelProperty> aProperties) throws BeanTableModelException
	{
		for (final TableModelProperty prop : aProperties)
		{
			if (prop.getter == null)
			{
				throw new BeanTableModelException(ERROR_3.format(new Object[] { prop.name }));
			}
			if (prop.isEditable && prop.setter == null)
			{
				throw new BeanTableModelException(ERROR_2.format(new Object[] { prop.name,
																				prop.getter.getName() }));
			}
		}
	}

	/**
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.table.TableModel#getColumnCount()
	 */
	public int getColumnCount()
	{
		return this.properties.length;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see tempo.impl.IBeanTableModel#getRowCount()
	 */
	public int getRowCount()
	{
		return this.array.size();
	}

	/**
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.table.TableModel#getValueAt(int, int)
	 */
	public Object getValueAt(final int rowIndex, final int columnIndex)
	{
		Object lReturn = null;
		try
		{
			this.lock.readLock().lock();
			lReturn = this.properties[columnIndex].getter.invoke(this.array.get(rowIndex), (Object[]) null);
		}
		catch (final IllegalArgumentException aException)
		{
			logger.error(ERROR_9.format(new Object[] {	this.properties[columnIndex].name,
														rowIndex,
														columnIndex }), aException);
		}
		catch (final IllegalAccessException aException)
		{
			logger.error(ERROR_9.format(new Object[] {	this.properties[columnIndex].name,
														rowIndex,
														columnIndex }), aException);
		}
		catch (final InvocationTargetException aException)
		{
			logger.error(ERROR_9.format(new Object[] {	this.properties[columnIndex].name,
														rowIndex,
														columnIndex }), aException);
		}
		finally
		{
			this.lock.readLock().unlock();
		}
		return lReturn;
	}

	/**
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.table.AbstractTableModel#getColumnClass(int)
	 */
	@Override
	public Class<?> getColumnClass(final int aColumnIndex)
	{
		return this.properties[aColumnIndex].getter.getReturnType();
	}

	/**
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.table.AbstractTableModel#getColumnName(int)
	 */
	@Override
	public String getColumnName(final int aColumn)
	{
		return this.properties[aColumn].label;
	}

	/**
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.table.AbstractTableModel#isCellEditable(int, int)
	 */
	@Override
	public boolean isCellEditable(final int aRowIndex, final int aColumnIndex)
	{
		return this.properties[aColumnIndex].isEditable;
	}

	/**
	 * This method is called from EDT !!! no not call him from another thread.
	 * 
	 * @see javax.swing.table.AbstractTableModel#setValueAt(java.lang.Object,
	 *      int, int)
	 */
	@Override
	public void setValueAt(final Object aValue, final int aRowIndex, final int aColumnIndex)
	{
		try
		{
			if (!EventQueue.isDispatchThread())
			{
				throw new IllegalAccessException("Can not call setValueAt from another thread that EDT. Calling thread is: "
						+ Thread.currentThread().getName());
			}

			this.lock.writeLock().lock();
			final int modelColIdx = aColumnIndex;// this.table.convertColumnIndexToModel(aColumnIndex);
			final int modelRowIdx = aRowIndex; // this.table.convertColumnIndexToModel(aRowIndex);
			final M toto = this.array.get(modelRowIdx);
			System.out.println("update: " + toto);
			this.properties[modelColIdx].setter.invoke(toto, aValue);
			this.fireBeanTableCellUpdated(aRowIndex, aColumnIndex);
		}
		catch (final IllegalArgumentException aException)
		{
			logger.error(ERROR_10.format(new Object[] { this.properties[aRowIndex].name,
														aRowIndex,
														aColumnIndex,
														aValue }), aException);
		}
		catch (final IllegalAccessException aException)
		{
			logger.error(ERROR_10.format(new Object[] { this.properties[aRowIndex].name,
														aRowIndex,
														aColumnIndex,
														aValue }), aException);
		}
		catch (final InvocationTargetException aException)
		{
			logger.error(ERROR_10.format(new Object[] { this.properties[aRowIndex].name,
														aRowIndex,
														aColumnIndex,
														aValue }), aException);
		}
		catch (final NullPointerException aException)
		{
			System.out.println(ERROR_10.format(new Object[] {	this.properties[aRowIndex].name,
																aRowIndex,
																aColumnIndex,
																aValue }));
			aException.printStackTrace();
		}
		finally
		{
			this.lock.writeLock().unlock();
		}
	}

	/**
	 * (non-Javadoc)
	 * 
	 * @see tempo.impl.IBeanTableModel#getValueAt(int)
	 */
	public final M getValueAt(final int rowIndex)
	{
		try
		{
			this.lock.readLock().lock();
			if (rowIndex >= 0 && rowIndex < this.array.size())
			{
				return this.array.get(rowIndex);
			}
		}
		finally
		{
			this.lock.readLock().unlock();
		}

		return null;
	}

	/**
	 * (non-Javadoc)
	 * 
	 * @see tempo.impl.IBeanTableModel#getIdxFor(M)
	 */
	public final int getIdxFor(final M aValue)
	{
		try
		{
			this.lock.readLock().lock();
			return this.arrayMap.get(aValue).intValue();
		}
		finally
		{
			this.lock.readLock().unlock();
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see tempo.impl.IBeanTableModel#getValueAt(M)
	 */
	public final M getValueAt(final M aRow)
	{
		final Integer value = getIdxFor(aRow);
		return value == null ? null : this.getValueAt(value.intValue());
	}

	/**
	 * (non-Javadoc)
	 * 
	 * @see tempo.impl.IBeanTableModel#add(M)
	 */
	@Override
	public boolean add(final M aValue)
	{
		try
		{
			this.lock.writeLock().lock();
			if (this.arrayMap.containsKey(aValue))
			{
				return false;
			}
			if (this.array.add(aValue))
			{
				this.arrayMap.put(aValue, this.array.size() - 1);
				fireBeanTableRowsInserted(this.array.size() - 1, this.array.size() - 1);
				return true;
			}
			else
			{
				logger.error("Failed to insert data at index:'" + this.array.size() + "', value: " + aValue);
				return false;
			}
		}
		finally
		{
			this.lock.writeLock().unlock();
		}
	}

	protected void fireBeanTableRowsInserted(final int aFirstRow, final int aLastRow)
	{
		fireTableRowsInserted(aFirstRow, aLastRow);
	}

	/**
	 * (non-Javadoc)
	 * 
	 * @see org.icroco.tablemodel.IBeanTableModel#addOrUpdateRow(java.lang.Object)
	 */
	@Override
	public boolean addOrUpdateRow(final M aValue)
	{
		try
		{
			this.lock.writeLock().lock();
			final Integer idx = this.arrayMap.get(aValue);

			if (idx == null)
			{
				return this.add(aValue);
			}
			this.array.set(idx.intValue(), aValue);
			fireBeanTableRowsUpdated(idx, idx);
			return true;
		}
		finally
		{
			this.lock.writeLock().unlock();
		}
	}

	protected void fireBeanTableRowsUpdated(final int aFirstRow, final int aLastRow)
	{
		fireTableRowsUpdated(aFirstRow, aLastRow);
	}

	/**
	 * (non-Javadoc)
	 * 
	 * @see tempo.impl.IBeanTableModel#updateRow(M, int, java.lang.Object,
	 *      boolean)
	 */
	@Override
	public void updateRow(Object aValue, M aRowKey, int aCol)
	{
		try
		{
			this.lock.writeLock().lock();
			final Integer row = this.arrayMap.get(aRowKey);
			if (row != null)
			{
				this.updateRow(aValue, row.intValue(), aCol);
			}
			else
			{
				if (logger.isDebugEnabled())
					logger.debug("Row not found: " + aRowKey);
			}
		}
		finally
		{
			this.lock.writeLock().unlock();
		}
	}

	/**
	 * (non-Javadoc)
	 * 
	 * @see tempo.impl.IBeanTableModel#updateRow(int, java.lang.Object, int)
	 */
	@Override
	public final M updateRow(final Object aValue, final int aRowIndex, final int aColumnIndex)
	{
		try
		{
			this.lock.writeLock().lock();
			final M record = this.array.get(aRowIndex);
			this.properties[aColumnIndex].setter.invoke(record, aValue);
			this.fireBeanTableCellUpdated(aRowIndex, aColumnIndex);
			return record;
		}
		catch (final IllegalArgumentException aException)
		{
			logger.error(ERROR_10.format(new Object[] { this.properties[aRowIndex].name,
														aRowIndex,
														aColumnIndex,
														aValue }), aException);
		}
		catch (final IllegalAccessException aException)
		{
			logger.error(ERROR_10.format(new Object[] { this.properties[aRowIndex].name,
														aRowIndex,
														aColumnIndex,
														aValue }), aException);
		}
		catch (final InvocationTargetException aException)
		{
			logger.error(ERROR_10.format(new Object[] { this.properties[aRowIndex].name,
														aRowIndex,
														aColumnIndex,
														aValue }), aException);
		}
		catch (final NullPointerException aException)
		{
			System.out.println(ERROR_10.format(new Object[] {	this.properties[aRowIndex].name,
																aRowIndex,
																aColumnIndex,
																aValue }));
			aException.printStackTrace();
		}
		finally
		{
			this.lock.writeLock().unlock();
		}
		return null;
	}

	protected void fireBeanTableCellUpdated(final int aRow, final int aColumn)
	{
		fireTableCellUpdated(aRow, aColumn);
	}

	/**
	 * @param model
	 * @param error
	 */
	protected static void dumpProperties(final BeanTableModel<?> model, final boolean error)
	{
		if (model != null)
		{
			final StringBuilder buffer = new StringBuilder(BUF_SIZE);
			buffer.append("Current Status: " + (model.clazz == null ? "null" : model.clazz.getName()) + "\n");
			int index = 0;
			if (model.properties != null)
			{
				for (final TableModelProperty prop : model.properties)
				{
					buffer.append("  * col=" + (index++) + " " + prop.toString() + "\n");
				}
			}
			if (error)
			{
				logger.error(buffer.toString());
			}
			else
			{
				logger.error(buffer.toString());
			}
		}
	}

	/**
	 * (non-Javadoc)
	 * 
	 * @see deai.ft.archi.common.swing.table.annotation.IBlinkTableModel#releaseFlash(deai.ft.archi.common.swing.table.annotation.DataChange,
	 *      boolean)
	 */
	// @Override
	// public void releaseFlash(final DataChange aChange, final boolean aNotify)
	// {
	// final int row = aChange.getRow();
	// if (row >= 0)
	// {
	// // xyState.add(change, Boolean.FALSE);
	// // System.out.println("notif:"+aNotify+" batch:"+batchChange+"
	// // change:"+aChange);
	// // System.out.println("fire view[:"+row+","+aChange.getColumn()+"
	// // notify["+aNotify+"]");
	// if (aNotify)
	// {
	// if (this.batchChange)
	// {
	// this.sharedModelEvent.set(row, aChange.getColumn());
	// this.fireTableChanged(this.sharedModelEvent);
	// } else
	// {
	// // if (isCellVisible(table, rowIndex, vColIndex))
	// this.fireTableCellUpdated(row, aChange.getColumn());
	// }
	// }
	// }
	// }
	/**
	 * clearFlashing make this job: stuspend flashing thread, clear queue event,
	 * redrow table, restart thread.
	 * 
	 */
	// public void clearFlashing()
	// {
	// if (this.updateThread != null)
	// {
	// this.updateThread.setUpdatesEnabled(false);
	// this.updateThread.getFlashQueue().clear();
	// // fireTableDataChanged();
	// this.updateThread.setUpdatesEnabled(true);
	// }
	// }
	// public void setBatchUpdates(final boolean batch)
	// {
	// this.batchChange = batch;
	// if (this.sharedModelEvent == null)
	// {
	// this.sharedModelEvent = new VisibleTableModelEvent(this);
	// }
	// this.sharedModelEvent.reset();
	// }
	/**
	 * setDefaultRenderer.
	 * 
	 * @param aTable
	 */
	// public void setDefaultRenderer(final JTable aTable)
	// {
	// this.table = aTable;
	// if (this.updateThread != null)
	// {
	// try
	// {
	// int i = 0;
	// for (final TableModelProperty onePH : this.properties)
	// {
	// // System.out.println("renderer: "+i+"
	// //
	// :"+table.getDefaultRenderer(onePH.propertyClass).getClass().getName());
	// // System.out.println("renderer: "+i+"
	// // :"+table.getColumnModel().getColumn(i).getCellRenderer());
	// if (onePH.blink)
	// {
	// TableCellRenderer oldRenderer = aTable.getCellRenderer(0, i);
	// if (oldRenderer == null)
	// {
	// oldRenderer = aTable.getDefaultRenderer(this.getColumnClass(i));
	// }
	// final RenderStage flash = new
	// JFlashTableCellRenderStage(this.updateThread.getFlashQueue(),
	// onePH.blinkBgColor);
	// if (oldRenderer instanceof PipelineTableCellRenderer)
	// {
	// ((PipelineTableCellRenderer) oldRenderer).addStage(flash);
	// // System.out.println("col: " +
	// // aTable.getColumnModel().getColumn(i).getHeaderValue()
	// // + " renderer: " +
	// // renderer);
	// } else
	// {
	// final RenderStage stages[] = new RenderStage[] { new
	// TableCellRendererStage(oldRenderer),
	// new JFlashTableCellRenderStage(this.updateThread.getFlashQueue(),
	// onePH.blinkBgColor) };
	// final PipelineTableCellRenderer pipeRenderer = new
	// PipelineTableCellRenderer(stages);
	// aTable.getColumnModel().getColumn(i).setCellRenderer(pipeRenderer);
	// // System.out.println("col: " +
	// // aTable.getColumnModel().getColumn(i).getHeaderValue()
	// // + " renderer: " +
	// // pipeRenderer);
	// }
	// }
	// i++;
	// }
	// } catch (final MissingResourceException exception)
	// {
	// exception.printStackTrace();
	// }
	// }
	// this.initOrgColumn();
	// this.table.getColumnModel().addColumnModelListener(new
	// TableColumnModelListener() {
	// public void columnAdded(final TableColumnModelEvent e)
	// {
	// BeanTableModel.this.initOrgColumn();
	// }
	//
	// public void columnRemoved(final TableColumnModelEvent e)
	// {
	// BeanTableModel.this.initOrgColumn();
	// }
	//
	// public void columnMoved(final TableColumnModelEvent e)
	// {
	// BeanTableModel.this.initOrgColumn();
	// }
	//
	// public void columnMarginChanged(final ChangeEvent e)
	// {
	// // nothing
	// }
	//
	// public void columnSelectionChanged(final ListSelectionEvent e)
	// {
	// // nothing
	// }
	// });
	// // TODO Auto-generated method stub
	//
	// }
	// final protected void initOrgColumn()
	// {
	// this.lock.writeLock().lock();
	// try
	// {
	// if (this.updateThread != null)
	// {
	// this.updateThread.setUpdatesEnabled(false);
	// this.updateThread.getFlashQueue().clear();
	// }
	// this.columnOrgPos = new int[this.properties.length];
	// for (int i = 0; i < this.table.getColumnModel().getColumnCount(); i++)
	// {
	// this.columnOrgPos[i] =
	// this.table.getColumnModel().getColumnIndex(this.properties[i].label);
	// // for (int i = 0; i < columnOrgPos.length ;i++)
	// // System.out.print(columnOrgPos[i]+",");
	// // System.out.println("");
	// }
	// } finally
	// {
	// this.lock.writeLock().unlock();
	// if (this.updateThread != null)
	// {
	// this.updateThread.setUpdatesEnabled(true);
	// }
	// }
	//
	// }
	// boolean isCellVisible(final JTable aTable, final int rowIndex, final int
	// vColIndex)
	// {
	// if (aTable == null || !(aTable.getParent() instanceof JViewport))
	// {
	// return false;
	// }
	// final JViewport viewport = (JViewport) aTable.getParent();
	//
	// final Rectangle rect = aTable.getCellRect(rowIndex, vColIndex, true);
	// // This rectangle is relative to the table where the
	// // northwest corner of cell (0,0) is always (0,0)
	// // if (aTable.getModel() != this || aTable.getModel() instanceof
	// // TableModelAddon)
	// // rect = aTable.getCellRect(((TableModelAddon)
	// // aTable.getModel()).getModelToView(rowIndex), vColIndex, true);
	// // else
	// // rect = aTable.getCellRect(rowIndex, vColIndex, true);
	//
	// // The location of the viewport relative to the table
	// final Point pt = viewport.getViewPosition();
	//
	// // Translate the cell location so that it is relative
	// // to the view, assuming the northwest corner of the
	// // view is (0,0)
	// rect.setLocation(rect.x - pt.x, rect.y - pt.y);
	//
	// // Check if view completely contains cell
	// this.cacheRectangle.setSize(viewport.getExtentSize());
	// return this.cacheRectangle.contains(rect);
	// }
	/*
	 * (non-Javadoc)
	 * 
	 * @see tempo.impl.IBeanTableModel#reset()
	 */
	public void reset()
	{
		try
		{
			this.lock.writeLock().lock();
			this.array.clear();
		}
		finally
		{
			this.lock.writeLock().unlock();
		}
		this.fireTableDataChanged();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see tempo.impl.IBeanTableModel#reset(java.util.Collection)
	 */
	public void reset(final Collection<M> collection)
	{
		try
		{
			this.lock.writeLock().lock();
			this.array.clear();
			this.array.addAll(collection);
		}
		finally
		{
			this.lock.writeLock().unlock();
		}
		this.fireTableDataChanged();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.icroco.tablemodel.IBeanTableModel#deleteRow(java.lang.Object)
	 */
	@Override
	public M deleteRow(M aValue)
	{
		try
		{
			this.lock.writeLock().lock();
			final Integer idx = this.arrayMap.get(aValue);
			if (idx != null)
				return deleteRowAt(idx.intValue());
		}
		finally
		{
			this.lock.writeLock().unlock();
		}

		return null;
	}

	/**
	 * (non-Javadoc)
	 * 
	 * @see org.icroco.tablemodel.IBeanTableModel#deleteRowAt(int)
	 */
	@Override
	public M deleteRowAt(int aIdx)
	{
		M remove = null;
		try
		{
			this.lock.writeLock().lock();
			if (aIdx >= this.array.size())
				return null;

			remove = this.array.get(aIdx);
			if (remove != null)
			{
				this.array.remove(aIdx);
				this.arrayMap.remove(remove);
			}
		}
		finally
		{
			this.lock.writeLock().unlock();
		}
		fireTableRowsDeleted(aIdx, aIdx);
		return remove;
	}

	/**
	 * (non-Javadoc)
	 * 
	 * @see org.icroco.tablemodel.IBeanTableModel#deleteRows(int, int)
	 */
	@Override
	public int deleteRows(int aStartIdx, int aEndIdx)
	{
		if (aStartIdx >= 0)
		{
			final int end = Math.min(this.array.size() - 1, aEndIdx);
			int i = aStartIdx;
			try
			{
				this.lock.writeLock().lock();
				for (; i <= end; i++)
				{
					this.arrayMap.remove(this.array.get(i));
				}
				for (int j = i - 1; j >= aStartIdx; j--)
				{
					this.array.remove(j);
				}
			}
			finally
			{
				this.lock.writeLock().unlock();
			}
			fireTableRowsDeleted(aStartIdx, end);
			return (i - aStartIdx);
		}
		return 0;
	}

	@Override
	public int[] deleteRows(final int... aIdx)
	{
		List<Integer> removed = new ArrayList<Integer>();
		final int[] newIdx = Arrays.copyOf(aIdx, aIdx.length);
		Arrays.sort(newIdx);
		try
		{
			this.lock.writeLock().lock();
			for (int i = newIdx.length - 1; i >= 0; i--)
			{
				if (newIdx[i] < this.array.size())
				{
					final M value = this.array.remove(newIdx[i]);
					if (value != null)
					{
						this.arrayMap.remove(value);
						removed.add(Integer.valueOf(newIdx[i]));
						// TODO is it right
						fireTableRowsDeleted(newIdx[i], newIdx[i]);
					}
				}
			}
		}
		finally
		{
			this.lock.writeLock().unlock();
		}

		Collections.sort(removed);
		final int[] lReturn = new int[removed.size()];
		for (int i = 0; i < removed.size(); i++)
		{
			lReturn[i] = removed.get(i).intValue();
		}
		return lReturn;
	}
	// public Rectangle getCellRect(final JTable aTable, final int row, final
	// int column, final boolean includeSpacing) {
	// Rectangle r = new Rectangle();
	// boolean valid = true;
	// if (row < 0) {
	// // y = height = 0;
	// valid = false;
	// }
	// else if (row >= getRowCount()) {
	// r.y = aTable.getHeight();
	// valid = false;
	// }
	// else {
	// r.height = aTable.getRowHeight(row);
	// r.y = (rowModel == null) ? row * r.height : rowModel.getPosition(row);
	// }
	//
	// if (column < 0) {
	// if( !aTable.getComponentOrientation().isLeftToRight() ) {
	// r.x = aTable.getWidth();
	// }
	// // otherwise, x = width = 0;
	// valid = false;
	// }
	// else if (column >= getColumnCount()) {
	// if( aTable.getComponentOrientation().isLeftToRight() ) {
	// r.x = aTable.getWidth();
	// }
	// // otherwise, x = width = 0;
	// valid = false;
	// }
	// else {
	// TableColumnModel cm = aTable.getColumnModel();
	// if( aTable.getComponentOrientation().isLeftToRight() ) {
	// for(int i = 0; i < column; i++) {
	// r.x += cm.getColumn(i).getWidth();
	// }
	// } else {
	// for(int i = cm.getColumnCount()-1; i > column; i--) {
	// r.x += cm.getColumn(i).getWidth();
	// }
	// }
	// r.width = cm.getColumn(column).getWidth();
	// }
	//
	// if (valid && !includeSpacing) {
	// // Bound the margins by their associated dimensions to prevent
	// // returning bounds with negative dimensions.
	// int rm = Math.min(aTable.getRowMargin(), r.height);
	// int cm = Math.min(aTable.getColumnModel().getColumnMargin(), r.width);
	// // This is not the same as grow(), it rounds differently.
	// r.setBounds(r.x + cm/2, r.y + rm/2, r.width - cm, r.height - rm);
	// }
	// return r;
	// }

}
