package org.icroco.tablemodel.impl;

import org.icroco.tablemodel.BeanTableModelException;
import org.icroco.tablemodel.TableModelFactory;

/**
 * package protected class to implements a {@link TableModelFactory}.
 * 
 * @author christophe
 * 
 */
public final class DefaultBeanTableModelFactory
{

	private DefaultBeanTableModelFactory()
	{
		super();

	}

	/**
	 * This is the better way to get a {@link BeanTableModel}. We make some
	 * stuff like JTable optimization for flashing, install renderer wrapper,
	 * ...
	 * 
	 * @param <B>
	 * @param aTable
	 * @param aClazz
	 * @return
	 * @throws BeanTableModelException
	 */
	public final static <B> BeanTableModel<B> createTableModel(final Class<B> aClazz) throws BeanTableModelException
	{
		if (aClazz == null)
			throw new IllegalArgumentException("Parameters Class can not be null");

		final BeanTableModel<B> model = new BeanTableModel<B>(aClazz);
		model.parseAnnotation();

		return model;
	}
}
