/**
 * 
 */
package org.icroco.tablemodel.impl.blinking;

import java.awt.Color;
import java.lang.reflect.Method;
import java.util.List;

import javax.swing.JTable;
import javax.swing.table.TableCellRenderer;

import org.icroco.tablemodel.BeanTableModelException;
import org.icroco.tablemodel.blinking.ABlinking;
import org.icroco.tablemodel.blinking.IBlinkTableModel;
import org.icroco.tablemodel.impl.BeanTableModel;
import org.icroco.tablemodel.impl.TableModelProperty;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * @author ouaibsky@gmail.com
 * 
 */
class BeanBlinkTableModel<M>
		extends BeanTableModel<M>
		implements IBlinkTableModel<M>
{
	/**
	 * serialVersionUID
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * Logger for this class
	 */
	private transient static Logger logger = LoggerFactory.getLogger(BeanBlinkTableModel.class);

	final TableFlasher flasher;

	/**
	 * @param aClass
	 * @throws BeanTableModelException
	 */
	public BeanBlinkTableModel(final JTable aTable, final Class<M> aClass) throws BeanTableModelException
	{
		super(aClass);
		this.flasher = new TableFlasher(aTable);
	}

	@Override
	protected TableModelProperty createNewProperty()
	{
		return new TableModelBlinkProperty();
	}

	/**
	 * (non-Javadoc)
	 * 
	 * @see org.icroco.tablemodel.impl.BeanTableModel#readAnnotationMethod(java.lang.Class)
	 */
	@Override
	protected List<TableModelProperty> readAnnotationMethod(Class<M> aClazz) throws BeanTableModelException
	{
		final List<TableModelProperty> props = super.readAnnotationMethod(aClazz);

		int i = -1;
		for (TableModelProperty prop : props)
		{
			i++;
			final TableModelBlinkProperty bProp = (TableModelBlinkProperty) prop;
			final Method setter = bProp.getSetter();
			if (setter != null)
			{
				final ABlinking blinkAnnotation = setter.getAnnotation(ABlinking.class);
				if (blinkAnnotation != null)
				{
					bProp.setBlink(true);
					bProp.setMode(blinkAnnotation.mode());
					bProp.setDuration(blinkAnnotation.duration());
					bProp.setRepeat(blinkAnnotation.repeat());
					bProp.setBlinkBgColor(Color.decode(blinkAnnotation.bgColor()));
					final String fgColor = blinkAnnotation.fgColor();
					if (fgColor != null && fgColor.trim().length() > 0)
						bProp.setBlinkFgColor(Color.decode((fgColor)));
				}
			}
		}

		return props;
	}

	/**
	 * package protected method, only called by 
	 */
	final void installRenderer()
	{
		int i = -1;
		for (TableModelProperty prop : this.properties)
		{
			i++;
			final TableModelBlinkProperty bProp = (TableModelBlinkProperty) prop;
			final JTable table = flasher.getTable();

			TableCellRenderer oldRenderer = null;

			// TODO check annotation.
			oldRenderer = table.getCellRenderer(0, i);
			if (oldRenderer == null)
			{
				oldRenderer = table.getDefaultRenderer(this.getColumnClass(i));
			}

			// Wrap the color renderer around the default renderer
			// TableCellRenderer colorRenderer = new
			// CellColorRenderer(defaultRenderer, provider);
			// Wrap the flash renderer around the colour renderer
			TableCellRenderer newRenderer = null; 
			// // Register our flash renderer with the table
			// table.setDefaultRenderer(klass, flashRenderer);

			switch (bProp.getMode())
			{
			case COLOR:
				newRenderer = new CellFlashColorRenderer(oldRenderer, this.flasher, bProp.getBlinkBgColor(), bProp.getBlinkFgColor());
				break;
			case BORDER:
				newRenderer = new CellFlashBorderRenderer(oldRenderer, this.flasher, bProp.getBlinkBgColor());
				break;
			case FONT:
				newRenderer = new CellFlashFontRenderer(oldRenderer, this.flasher, bProp.getBlinkFgColor());
				break;

			default:
				logger.error("unknown blinking mode: " + bProp.getMode());
				break;
			}
			if (newRenderer != null)
				table.getColumnModel().getColumn(i).setCellRenderer(newRenderer);

		}

	}

	@Override
	protected void fireBeanTableCellUpdated(int aRow, int aColumn)
	{
		final TableModelBlinkProperty prop = (TableModelBlinkProperty) this.properties[aColumn];
		if (prop.isBlink())
			this.flasher.flashCell(aRow, aColumn, prop.getRepeat(), prop.getDuration());
		else
			fireTableCellUpdated(aRow, aColumn);
	}

	// @Override
	// protected void fireBeanTableRowsInserted(int aFirstRow, int aLastRow)
	// {
	// // TODO Auto-generated method stub
	// super.fireBeanTableRowsInserted(aFirstRow, aLastRow);
	// }
	//	
	// @Override
	// protected void fireBeanTableRowsUpdated(int aFirstRow, int aLastRow)
	// {
	// // TODO Auto-generated method stub
	// super.fireBeanTableRowsUpdated(aFirstRow, aLastRow);
	// }
}
