package org.icroco.tablemodel.impl.blinking;

/**
 * CellFlashBorderRenderer
 *
 * Specialised renderer class for giving a flashing cell a 3D effect
 *
 * @author L.D'Abreo
 *
 * @see TableCellRenderer
 * @see JTable
 * @see FlashProvider
 */
import java.awt.Color;
import java.awt.Component;

import javax.swing.AbstractButton;
import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JTable;
import javax.swing.UIManager;
import javax.swing.border.Border;
import javax.swing.border.EmptyBorder;
import javax.swing.table.TableCellRenderer;

import org.icroco.tablemodel.blinking.ABlinking;
import org.icroco.tablemodel.blinking.CellFlashProvider;

public class CellFlashBorderRenderer extends AbstractCellFlashRenderer
    implements TableCellRenderer
{
    
    protected final Border raised;
    protected static final Border noFocusBorder = new EmptyBorder(1, 1, 1, 1);
    protected static final Border focusBorder   = UIManager.getBorder("Table.focusCellHighlightBorder");
    
    public CellFlashBorderRenderer(final TableCellRenderer aDelegate, final CellFlashProvider aProvider, final Color aBorderColor)
    {
    	super(aDelegate, aProvider);
    	this.raised = BorderFactory.createLineBorder(aBorderColor);
    }

    public CellFlashBorderRenderer(final TableCellRenderer aDelegate, final CellFlashProvider aProvider)
    {
    	this(aDelegate, aProvider, Color.decode(ABlinking.DEFAULT_BG_COLOR));
    }
    
    public Component getTableCellRendererComponent(final JTable aTable, final Object value, final boolean isSelected,
                                                   final boolean hasFocus, final int row, final int column)
    {
        
        // Get the component from the delegate
        final Component c = this.delegate.getTableCellRendererComponent(aTable, value, isSelected, hasFocus, row, column);
        
        // Convert view column to model column
        final int mCol = aTable.convertColumnIndexToModel(column);
        final int mRow = aTable.convertRowIndexToModel(row);

        // invert the colours to flash
        final JComponent jc = (JComponent) c;
        
        // If an abstract button e.g JCheckBox then set
        // border painted to true
        if (jc instanceof AbstractButton)
        {
            ((AbstractButton) jc).setBorderPainted(true);
        }
        if (this.provider.isFlashOn(mRow, mCol))
        {
            jc.setBorder(raised);
        }
        else
        {
            if (hasFocus)
            {
                jc.setBorder(focusBorder);
            }
            else
            {
                jc.setBorder(noFocusBorder);
            }
        }
        
        return c;
        
    }
    
}