package org.icroco.tablemodel.impl.blinking;

/**
 * CellFlashColorRenderer
 *
 * Specialised renderer class for flashing the colors of a cell
 *
 * @author L.D'Abreo
 *
 * @see TableCellRenderer
 * @see JTable
 * @see CellFlashProvider
 */
import java.awt.Color;
import java.awt.Component;

import javax.swing.JTable;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableCellRenderer;

import org.icroco.tablemodel.blinking.CellFlashProvider;

/**
 * CellFlashColorRenderer.
 * 
 * @author clalleme - 2 juin 08
 *
 *
 * <!-- $Id: PreTradeEclipseCodeTemplates.xml,v 1.8 2007/03/12 12:35:12 anavarro Exp $ -->.
 *
 */
public class CellFlashColorRenderer
    extends AbstractCellFlashRenderer
{
	final Color bgrd;
	final Color fgrd;

	public CellFlashColorRenderer(TableCellRenderer aDelegate, CellFlashProvider aProvider, final Color aBackground, final Color aForeground)
	{
		super(aDelegate, aProvider);
		this.fgrd = aForeground;
		this.bgrd = aBackground;
	}

	public CellFlashColorRenderer(TableCellRenderer aDelegate, CellFlashProvider aProvider, final Color aBackground)
	{
		this(aDelegate, aProvider, aBackground, null);
	}
	/**
	 * @param aDelegate
	 * @param aProvider
	 */
	public CellFlashColorRenderer(TableCellRenderer aDelegate, CellFlashProvider aProvider)
	{
		this(aDelegate, aProvider,Color.RED);
	}

	/** 
     * (non-Javadoc)
     *
     * @see javax.swing.table.TableCellRenderer#getTableCellRendererComponent(javax.swing.JTable, java.lang.Object, boolean, boolean, int, int)
     */
    @Override
    public Component getTableCellRendererComponent(final JTable aTable, final Object aValue, final boolean aIsSelected,
                                                   final boolean aHasFocus, final int aRow, final int aCol)
    {
    	// Get the component from the delegate
    	if (this.delegate instanceof DefaultTableCellRenderer)
    	{
    		((DefaultTableCellRenderer) this.delegate).setBackground(null);
    		((DefaultTableCellRenderer) this.delegate).setForeground(null);
    	}
        final Component c = this.delegate.getTableCellRendererComponent(aTable, aValue, aIsSelected, aHasFocus, aRow, aCol);
        
        // Convert view column to model column
        final int mCol = aTable.convertColumnIndexToModel(aCol);
        final int mRow = aTable.convertRowIndexToModel(aRow);
        // invert the colors to flash
        if (this.provider.isFlashOn(mRow, mCol))
        {
            if (this.bgrd != null)
            	c.setBackground(this.bgrd);
            if (this.fgrd != null)
            	c.setForeground(this.fgrd);
        }
        
        return c;
        
    }
    
}