package org.icroco.tablemodel.impl.blinking;

/**
 * CellFlashFontRenderer
 *
 * Specialised renderer class for flashing the font of a cell
 *
 * @author L.D'Abreo
 *
 * @see TableCellRenderer
 * @see JTable
 * @see CellFlashProvider
 */
import java.awt.Color;
import java.awt.Component;
import java.awt.Font;

import javax.swing.JComponent;
import javax.swing.JTable;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableCellRenderer;

import org.icroco.tablemodel.blinking.CellFlashProvider;

public class CellFlashFontRenderer
    extends AbstractCellFlashRenderer
{
    final static int style = Font.BOLD | Font.ITALIC;
    final Color fgrd;
    
    /**
	 * @param aDelegate
	 * @param aProvider
	 */
	public CellFlashFontRenderer(TableCellRenderer aDelegate, CellFlashProvider aProvider, final Color aFgColor)
	{
		super(aDelegate, aProvider);
		this.fgrd = aFgColor;
	}

	public Component getTableCellRendererComponent(final JTable aTable, final Object aValue, final boolean aIsSelected,
                                                   final boolean aHasFocus, final int aRow, final int aColumn)
    {
    	if (this.delegate instanceof DefaultTableCellRenderer)
    	{
    		((DefaultTableCellRenderer) this.delegate).setForeground(null);
    	}

        // Get the component from the delegate
        final Component c = this.delegate.getTableCellRendererComponent(aTable, aValue, aIsSelected, aHasFocus, aRow, aColumn);
        
        // Convert view column to model column
        final int mCol = aTable.convertColumnIndexToModel(aColumn);
        final int mRow = aTable.convertRowIndexToModel(aRow);
        
        // italic and bold the Font to flash
        final JComponent jc = (JComponent) c;
        final Font f = jc.getFont();
        
        if (this.provider.isFlashOn(mRow, mCol))
        {
            jc.setFont(f.deriveFont(style));
//            if (this.fgrd != null)
//            	c.setForeground(this.fgrd);
        }
        else
        {
            jc.setFont(f.deriveFont(Font.PLAIN));
        }
        return c;
        
    }
    
}
